import argparse

from monitor.sys_info import SysInfo
from monitor.monitor_pids import *
from monitor.get_process_name import *
import asyncio


def main():
    parser = argparse.ArgumentParser(description='指定进程名或进程Pid即可获取性能数据~')
    parser.add_argument('-n', '--name', required=True, help='进程名称')
    parser.add_argument('-t', '--time', help='采集间隔时间/s', type=float, default=1)
    parser.add_argument('-i', '--info', help='输出当前系统使用情况', action='store_true')
    args = parser.parse_args()
    print(args)

    if args.info:
        print(SysInfo().get_device_info())
        exit()
    process = None
    if args.name:
        process = args.name
    else:
        parser.print_help()
        exit()

    if args.time:
        interval = args.time

    # process = 'YunXiu-PC'
    # interval = 1
    process_name = ProcessName()
    if process is not None:
        print('监测进程<{}>，采集数据间隔<{}s>'.format(process, interval))
    else:
        parser.print_help()
        exit()

    pid = process_name.find_main_process_pid(process)
    if process is None:
        process = process_name.get_pid_find_pid(int(pid))

    if pid is None or process is None:
        print('进程列表中未找到<{}>，请检查应用是否启动！停止监测任务'.format(process))
        sys.exit(0)

    print('开始监测进程<{}>，主进程pid<{}>'.format(process, pid))
    pidsPerf = PidsPerf(process, interval)
    asyncio.run(pidsPerf.start_perf())


if __name__ == '__main__':
    main()



import platform

try:
    import objc
    import Foundation
except:
    pass

if platform.system() != 'Windows':
    IOKit = Foundation.NSBundle.bundleWithIdentifier_("com.apple.framework.IOKit")

    functions = [
        ("IOServiceGetMatchingService", b"II@"),
        ("IOServiceMatching", b"@*"),
        ("IORegistryEntryCreateCFProperties", b"IIo^@@I"),
    ]

    objc.loadBundleFunctions(IOKit, globals(), functions)


    def accelerator_performance_statistics() -> dict[str, int]:
        accelerator_info = IOServiceGetMatchingService(
            0, IOServiceMatching(b"IOAccelerator")
        )
        if accelerator_info == 0:
            raise RuntimeError("IOAccelerator not found")
        # else...
        err, props = IORegistryEntryCreateCFProperties(accelerator_info, None, None, 0)
        if err != 0:
            raise RuntimeError("IOAccelerator properties not found")
        # else...
        return dict(props)


    def get_gpu_memory():
        try:
            gpu_info = accelerator_performance_statistics()
            use_gpu_memory = 0
            total_gpu_memory = 0
            free_gpu_memory = 0
            for key, value in gpu_info["PerformanceStatistics"].items():
                if key == "In use system memory" or key == "gartUsedBytes":
                    use_gpu_memory = int(value)
                elif key == "gartSizeBytes":
                    total_gpu_memory = int(value)
                elif key == "gartFreeBytes":
                    free_gpu_memory = int(value)
                elif key == "Alloc system memory":
                    total_gpu_memory = int(value)
                    # print('device usage', device_usage)
            # use_gpu_memory = gpu_info["PerformanceStatistics"]["In use system memory"]
            # total_gpu_memory = gpu_info["PerformanceStatistics"]["Alloc system memory"]
        except Exception as e:
            print(f"Mac端在arm64平台通过接口获取Gpu Memory失败：{e}")
            use_gpu_memory = 0
            total_gpu_memory = 0
            free_gpu_memory = 0
        if free_gpu_memory == 0:
            free_gpu_memory = total_gpu_memory - use_gpu_memory
        return round(use_gpu_memory / 1024 / 1024, 2), round(total_gpu_memory / 1024 / 1024, 2), round(
            free_gpu_memory / 1024 / 1024, 2)



import subprocess
import threading
import time
import json
import re
import os


class VmMapMemory:
    def __init__(self):
        """
        初始化实时内存监控器
        """
        self.pids = set()  # 当前监控的 PID
        self.current_data = {}  # 存储最新的内存数据 {pid: memory_mb}
        self.lock = threading.Lock()  # 线程锁
        self.running = True  # 控制线程运行状态
        self.thread = None  # 监控线程
        # 启动监控线程
        self.start_monitoring()

    def start_monitoring(self):
        """启动监控线程"""
        self.thread = threading.Thread(target=self._monitor_memory)
        self.thread.daemon = True  # 设置为守护线程
        self.thread.start()

    def update_pids(self, new_pids):
        """更新要监控的 PID 列表"""
        with self.lock:
            # 转换为字符串并更新集合
            self.pids = set(map(str, new_pids))

    def _convert_to_mb(self, mem_str):
        """
        将内存字符串转换为 MB
        支持格式: 123, 123K, 123M, 123G, 12G+, 12G-
        """
        # 移除逗号（如果存在）
        mem_str = mem_str.replace(',', '')

        # 检查单位并转换
        if 'G' in mem_str:
            return float(mem_str.replace('G', '').rstrip('+-')) * 1024
        elif 'M' in mem_str:
            return float(mem_str.replace('M', '').rstrip('+-'))
        elif 'K' in mem_str:
            return float(mem_str.replace('K', '').rstrip('+-')) / 1024
        else:
            # 假设为字节
            return float(mem_str.rstrip('+-')) / (1024 * 1024)

    def _monitor_memory(self):
        """线程函数：实时监控内存使用"""
        # macOS 专用 top 命令
        cmd = ["top", "-stats", "pid,mem"]

        # 启动 top 进程
        process = subprocess.Popen(
            cmd,
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE,
            text=True,
            bufsize=1,  # 行缓冲
            universal_newlines=True  # 确保文本模式
        )

        try:
            while self.running:
                line = process.stdout.readline()  # 逐行读取

                if not line:  # 进程结束或无输出时退出
                    break

                # 匹配 PID 和内存行 (例如: "12345  123.4M")
                match = re.match(r'^\s*(\d+)\s+([\d,.]+[KMG][+-]?)\s*$', line.strip())
                if match:
                    pid = match.group(1)
                    mem_str = match.group(2)

                    # 检查是否在监控列表中
                    with self.lock:
                        if pid in self.pids:
                            mem_mb = self._convert_to_mb(mem_str)
                            # 直接更新当前数据，不保存历史
                            self.current_data[pid] = round(mem_mb, 2)

        except Exception as e:
            print(f"Monitoring error: {e}")
        finally:
            if process.poll() is None:  # 确保进程终止
                process.terminate()

    def get_current_data(self):
        """
        获取当前内存数据（线程安全）
        只返回当前监控的 PID 的最新内存值
        """
        with self.lock:
            # 只返回当前监控列表中的 PID 数据
            return {
                pid: self.current_data.get(pid, 0.0)
                for pid in self.pids
            }

    def get_current_json(self):
        """获取 JSON 格式的当前内存数据"""
        data = self.get_current_data()
        return json.dumps(data, indent=2)

    def stop(self):
        """停止监控"""
        self.running = False
        if self.thread and self.thread.is_alive():
            self.thread.join(timeout=1)
        print("Memory monitor stopped")


import asyncio
import time

try:
    from monitor.get_process_name import *
except:
    from get_process_name import *
import psutil
import subprocess


class MacPerformance:

    def __init__(self, pid, interval=1):
        super(MacPerformance, self).__init__()
        if pid is None or not psutil.pid_exists(pid):
            print(f'进程{pid}不存在，停止监测')
            return
        self.interval = interval
        self.pid = pid
        self.process = self.get_process(pid)
        self.process_cpu_percent = 0
        self.system_cpu_percent = 0
        self.real_mem = 0
        self.private_mem = 0
        self.mem_percent = 0
        self.thread_count = 0
        self.io_read_bytes = 0
        self.io_write_bytes = 0
        self.header = ['时间',
                       'App CPU/%',
                       'System CPU/%',
                       '专用内存/MB',
                       '实际内存/MB',
                       'App内存使用率/%',
                       '线程数',
                       'IO(Read)/kb',
                       'IO(Write)/kb']

    def get_all_data(self):
        if self.process is not None:
            self.get_system_cpu_percent_usage()  # system cpu
            self.get_app_cpu_percent_usage()  # app cpu
            self.get_app_memory_usage()  # app memory
            io_read_bytes_1, io_write_bytes_1 = self.get_disk_usage()  # io read/write
            time.sleep(self.interval)
            io_read_bytes_2, io_write_bytes_2 = self.get_disk_usage()  # io read/write
            self.io_read_bytes = round((io_read_bytes_2 - io_read_bytes_1) / 1024, 2)  # io read/kb
            self.io_write_bytes = round((io_write_bytes_2 - io_write_bytes_1) / 1024, 2)  # io write/kb

    def get_system_cpu_percent_usage(self):
        '''
        获取系统cpu占比
        :return:
        '''
        system_times = psutil.cpu_times_percent()
        self.system_cpu_percent = round(100 - system_times.idle, 2)
        return self.system_cpu_percent

    def get_app_cpu_percent_usage(self):
        '''
        获取进程cpu占比
        :param pid:
        :return:
        '''
        try:
            self.process_cpu_percent = self.process.cpu_percent()
            return self.process_cpu_percent
        except (psutil.NoSuchProcess, psutil.AccessDenied, psutil.ZombieProcess):
            return 0

    def get_app_memory_usage(self):
        '''
        获取进程的内存使用情况
        :return:
        '''
        process_memory = self.process.memory_info()
        self.mem_percent = u'%.2f' % (self.process.memory_percent())  # 内存利用率
        self.real_mem = round(process_memory.rss / 1024 / 1024, 2)  # 实际内存
        self.private_mem = self.get_private_memory()  # 专用内存/MB
        self.thread_count = self.process.num_threads()  # 线程总数
        return self.real_mem, self.private_mem, self.mem_percent, self.thread_count

    def get_private_memory(self):
        try:
            command = "top -pid {} -l 1 | tail -n 1 | awk '{{print $8}}'".format(self.pid)
            memory_info = subprocess.check_output(command, shell=True).decode().strip().replace('M', '')
            return int(memory_info)
        except:
            return 0

    def get_disk_usage(self):
        '''
        获取进程磁盘读写情况
        :return:
        '''
        if self.process is not None:
            try:
                io_counters = psutil.disk_io_counters()
                io_read_bytes = io_counters.read_bytes
                io_write_bytes = io_counters.write_bytes
                return io_read_bytes, io_write_bytes
            except (psutil.NoSuchProcess, psutil.AccessDenied, psutil.ZombieProcess):
                return 0, 0

    async def get_all_monitor(self):
        await asyncio.gather(self.get_all_data())
        return self.get_all_usage()

    def get_all_usage(self):
        return [
            self.process_cpu_percent,  # 0
            self.system_cpu_percent,  # 1
            self.private_mem,  # 2
            self.real_mem,  # 3
            self.mem_percent,  # 4
            self.thread_count,  # 5
            self.io_read_bytes,  # 7
            self.io_write_bytes  # 8
        ]

    def get_process(self, pid):
        '''
        通过pid实例化process
        :param pid:
        :return:
        '''
        if pid is None:
            print(f'进程{pid}为None')
            return None
        if psutil.pid_exists(pid):
            process = psutil.Process(pid)
            return process
        else:
            print(f'进程{pid}不存在')
            return None



import ctypes
from ctypes import wintypes, byref, windll

import psutil

# 定义以及初始化 API 函数和类型
GetSystemTimes = ctypes.windll.kernel32.GetSystemTimes
GetProcessTimes = ctypes.windll.kernel32.GetProcessTimes
OpenProcess = ctypes.windll.kernel32.OpenProcess
GetProcessIoCounters = ctypes.windll.kernel32.GetProcessIoCounters
# 权限
PROCESS_QUERY_INFORMATION = 0x0400
PROCESS_VM_READ = 0x0010
GetCurrentProcess = ctypes.windll.kernel32.GetCurrentProcess
GetCurrentProcessId = ctypes.windll.kernel32.GetCurrentProcessId
CloseHandle = ctypes.windll.kernel32.CloseHandle


class IO_COUNTERS(ctypes.Structure):
    _fields_ = [('ReadOperationCount', ctypes.c_ulonglong),
                ('WriteOperationCount', ctypes.c_ulonglong),
                ('OtherOperationCount', ctypes.c_ulonglong),
                ('ReadTransferCount', ctypes.c_ulonglong),
                ('WriteTransferCount', ctypes.c_ulonglong),
                ('OtherTransferCount', ctypes.c_ulonglong)]


class FILETIME(ctypes.Structure):
    _fields_ = [("dwLowDateTime", wintypes.DWORD),
                ("dwHighDateTime", wintypes.DWORD)]


# 定义保存进程内存信息结果
class PROCESS_MEMORY_COUNTERS_EX(ctypes.Structure):
    _fields_ = [
        ('cb', wintypes.DWORD),
        ('PageFaultCount', wintypes.DWORD),
        ('PeakWorkingSetSize', ctypes.c_size_t),
        ('WorkingSetSize', ctypes.c_size_t),
        ('QuotaPeakPagedPoolUsage', ctypes.c_size_t),
        ('QuotaPagedPoolUsage', ctypes.c_size_t),
        ('QuotaPeakNonPagedPoolUsage', ctypes.c_size_t),
        ('QuotaNonPagedPoolUsage', ctypes.c_size_t),
        ('PagefileUsage', ctypes.c_size_t),
        ('PeakPagefileUsage', ctypes.c_size_t),
        ('PrivateUsage', ctypes.c_size_t),
    ]


PROCESS_QUERY_INFORMATION = 0x0400
PROCESS_VM_READ = 0x0010

# 定义关闭句柄所需的Windows函数
CloseHandle = windll.kernel32.CloseHandle
CloseHandle.restype = wintypes.BOOL
CloseHandle.argtypes = [wintypes.HANDLE]

# 函数签名
GetSystemTimes.restype = wintypes.BOOL
GetProcessTimes.restype = wintypes.BOOL
OpenProcess.restype = wintypes.HANDLE
CloseHandle.restype = wintypes.BOOL
GetProcessTimes.argtypes = [
    wintypes.HANDLE,  # hProcess
    ctypes.POINTER(FILETIME),  # lpCreationTime
    ctypes.POINTER(FILETIME),  # lpExitTime
    ctypes.POINTER(FILETIME),  # lpKernelTime
    ctypes.POINTER(FILETIME)  # lpUserTime
]


class PidWinPerformance:

    def filetime_to_100nano_seconds(self, filetime):
        return (filetime.dwHighDateTime << 32) + filetime.dwLowDateTime

    def get_process_cpu_time(self, pid):
        process_handle = ctypes.windll.kernel32.OpenProcess(PROCESS_QUERY_INFORMATION, False, pid)
        if not process_handle:
            return None

        kernel_time = FILETIME()
        user_time = FILETIME()
        if not ctypes.windll.kernel32.GetProcessTimes(process_handle,
                                                      byref(FILETIME()),  # ignore creation_time
                                                      byref(FILETIME()),  # ignore exit_time
                                                      byref(kernel_time),
                                                      byref(user_time)):
            ctypes.windll.kernel32.CloseHandle(process_handle)
            return None

        ctypes.windll.kernel32.CloseHandle(process_handle)
        return self.filetime_to_100nano_seconds(kernel_time) + self.filetime_to_100nano_seconds(user_time)

    def get_process_memory(self, pid):
        if not psutil.pid_exists(pid):
            return 0
        OpenProcess = ctypes.windll.kernel32.OpenProcess
        OpenProcess.restype = wintypes.HANDLE
        OpenProcess.argtypes = [wintypes.DWORD, ctypes.wintypes.BOOL, wintypes.DWORD]
        GetProcessMemoryInfo = ctypes.windll.psapi.GetProcessMemoryInfo
        GetProcessMemoryInfo.argtypes = [wintypes.HANDLE, ctypes.POINTER(PROCESS_MEMORY_COUNTERS_EX), wintypes.DWORD]
        GetProcessMemoryInfo.restype = ctypes.wintypes.BOOL
        process = OpenProcess(PROCESS_QUERY_INFORMATION | PROCESS_VM_READ, False, pid)
        if psutil.Process(pid).is_running() and psutil.Process(pid).status() != psutil.STATUS_ZOMBIE:
            if not process:
                raise ctypes.WinError()
            counters = PROCESS_MEMORY_COUNTERS_EX()
            counters.cb = ctypes.sizeof(PROCESS_MEMORY_COUNTERS_EX)

            if not GetProcessMemoryInfo(process, ctypes.byref(counters), counters.cb):
                CloseHandle(process)
                raise ctypes.WinError()
            CloseHandle(process)
            return counters
        return 0

    def get_io_counters(self, pid):
        counters = IO_COUNTERS()
        hProcess = OpenProcess(PROCESS_QUERY_INFORMATION, False, pid)
        if hProcess == 0:
            error_code = ctypes.windll.kernel32.GetLastError()
            print(f"Error opening process (code: {error_code})")
            return None
        if not GetProcessIoCounters(hProcess, ctypes.byref(counters)):
            error_code = ctypes.windll.kernel32.GetLastError()
            print(f"Error getting IO counters (code: {error_code})")
            return None
        ctypes.windll.kernel32.CloseHandle(hProcess)
        return counters


# import time
# from monitor.get_process import *
#
# # 示例使用：
# system_time_pre = PidWinPerformance().filetime_to_100nano_seconds(FILETIME())
#
# monitor_duration_seconds = 60  # 监控持续时长（秒）
# monitor_interval_seconds = 1  # 监控间隔（秒）
# # 开始监控
# end_time = time.time() + monitor_duration_seconds
#
# app_name = "chrome.exe"
# ppid = find_main_process_pid(app_name)
# child_pids = get_children_pids(ppid)
# child_pids.add(ppid)
# print('pid', child_pids)
# process_cpu_times_pre = {pid: PidWinPerformance().get_process_cpu_time(pid) for pid in child_pids}
# process_io_counters_pre = {pid: PidWinPerformance().get_io_counters(pid) for pid in child_pids}
# while True:
#     start_time = time.time()  # 记录循环开始的时间
#     system_idle_time = FILETIME()
#     system_kernel_time = FILETIME()
#     system_user_time = FILETIME()
#     ctypes.windll.kernel32.GetSystemTimes(byref(system_idle_time),
#                                           byref(system_kernel_time),
#                                           byref(system_user_time))
#     system_time_post = PidWinPerformance().filetime_to_100nano_seconds(
#         system_kernel_time) + PidWinPerformance().filetime_to_100nano_seconds(system_user_time)
#
#     child_pids = get_children_pids(ppid)
#     child_pids.add(ppid)
#     print('pid', child_pids)
#     new_pids_cpu_time_pre = {pid: PidWinPerformance().get_process_cpu_time(pid) for pid in child_pids if
#                              pid not in process_cpu_times_pre}
#     process_cpu_times_pre.update(new_pids_cpu_time_pre)
#
#     print('----------------------------------------------------------')
#     for pid in child_pids:
#         process_cpu_time_post = PidWinPerformance().get_process_cpu_time(pid)
#         memory_info = PidWinPerformance().get_process_memory(pid)
#         # 获取磁盘I/O信息
#         io_counters = PidWinPerformance().get_io_counters(pid)
#         if process_cpu_time_post:
#             # 计算CPU使用率
#             cpu_usage = (process_cpu_time_post - process_cpu_times_pre[pid]) / (
#                     system_time_post - system_time_pre) * 100
#             print(
#                 f"({time.strftime('%H:%M:%S')}) PID {pid}: CPU Usage = {cpu_usage:.2f}%， Memory Info:{memory_info.WorkingSetSize},{memory_info.PrivateUsage}")
#             # 更新之前的CPU时间
#             process_cpu_times_pre[pid] = process_cpu_time_post
#         else:
#             print(f"PID {pid}: Process not found or access denied")
#
#         if io_counters and pid in process_io_counters_pre:
#             # 计算I/O变化量
#             read_bytes_sec = (io_counters.ReadTransferCount - process_io_counters_pre[
#                 pid].ReadTransferCount) / monitor_interval_seconds
#             write_bytes_sec = (io_counters.WriteTransferCount - process_io_counters_pre[
#                 pid].WriteTransferCount) / monitor_interval_seconds
#             print(f"PID {pid}: Read Bytes/sec = {read_bytes_sec}, Write Bytes/sec = {write_bytes_sec}")
#             # 更新之前的I/O计数器
#             process_io_counters_pre[pid] = io_counters
#         else:
#             print(f"PID {pid}: Could not get IO counters")
#
#     # 更新系统时间
#     system_time_pre = system_time_post
#     # 等待以确保大约每秒更新一次
#     elapsed = time.time() - start_time
#     time.sleep(max(0, monitor_interval_seconds - elapsed))
# print("监控完毕。")



import psutil as ps
import socket
import uuid
import datetime


class SysInfo:

    def get_mac_address(self):
        mac = uuid.UUID(int=uuid.getnode()).hex[-12:]
        return ':'.join([mac[e:e + 2] for e in range(0, 11, 2)])

    def get_device_info(self):
        # 主机名
        host_name = socket.gethostname()
        # ip地址
        ip = self.getIP()
        # 系统用户
        users_list = ','.join([u.name for u in ps.users()])
        # 系统启动时间
        start_time = datetime.datetime.fromtimestamp(ps.boot_time()).strftime('%Y-%m-%d %H:%M:%S')
        sys_info = {'HostName': host_name,
                    'IP': ip,
                    'MAC': self.get_mac_address(),
                    'User': users_list,
                    'Start Time': start_time}

        # CPU物理核心
        cpu_count = ps.cpu_count(logical=False)
        # CPU逻辑数量
        cpu_logical = ps.cpu_count()
        cpu_info = {'Cpu Amount': cpu_count,
                    'Cpu Logical': cpu_logical}

        # 内存信息
        mem = ps.virtual_memory()
        # 内存总量
        mem_total = round(mem.total / 1024 / 1024 / 1024, 2)
        # 内存使用量
        mem_used = round(mem.used / 1024 / 1024 / 1024, 2)
        # 内存可用量
        mem_available = round(mem.available / 1024 / 1024 / 1024, 2)
        # 内存使用百分比
        mem_percent = mem.percent
        mem_info = {'Total Memory': str(mem_total) + 'GB',
                    'Memory Used': str(mem_used) + 'GB',
                    'Memory Free': str(mem_available) + 'GB',
                    'Memory Usage': str(mem_percent) + '%'}

        # 磁盘信息
        io = ps.disk_partitions()
        disk_info = []
        for i in io:
            disk = ps.disk_usage(i.mountpoint)
            disk_data = {'disk_name': i.mountpoint,
                         'total': str(round(disk.total / 1024 / 1024 / 1024, 1)) + 'GB',
                         'used': str(round(disk.used / 1024 / 1024 / 1024, 1)) + 'GB',
                         'surplus': str(round(disk.free / 1024 / 1024 / 1024, 1)) + 'GB',
                         'rate': str(ps.disk_usage(i.mountpoint).percent) + '%'}
            disk_info.append(disk_data)

        # 网卡信息
        net = ps.net_io_counters()
        bytes_sent = '{0:.2f}'.format(net.bytes_sent / 1024 / 1024)
        bytes_rcvd = '{0:.2f}'.format(net.bytes_recv / 1024 / 1024)
        net_info = {'bytes_sent': bytes_sent + 'MB',
                    'bytes_rcvd': bytes_rcvd + 'MB'}

        data = {'sys_info': sys_info, 'cpu_info': cpu_info,
                'mem_info': mem_info, 'disk_info': disk_info,
                'net_info': net_info}
        return data

    def getIP(self):
        """获取ipv4地址"""
        dic = ps.net_if_addrs()
        ipv4_list = []
        for adapter in dic:
            snicList = dic[adapter]
            for snic in snicList:
                if snic.family.name == 'AF_INET':
                    ipv4 = snic.address
                    if ipv4 != '127.0.0.1':
                        ipv4_list.append(ipv4)
        if len(ipv4_list) >= 1:
            return ipv4_list[0]
        else:
            return 'None'




import psutil


class ProcessName:

    def get_ios_process_name(self) -> list:
        '''
        Mac端获取正常运行的进程列表名称
        :return:
        '''
        process_names = []
        for process in psutil.process_iter():
            if process.ppid() == 1:
                if 'apple' not in process.name().lower():
                    process_names.append(process.name())
        return process_names

    def get_win_process_name(self):
        '''
        Windows端获取正常运行的进程列表名称
        :return:
        '''
        processes = {}
        for process in psutil.process_iter():
            try:
                pinfo = process.as_dict(attrs=['pid', 'name', 'username', 'memory_info', 'ppid'])
                processes[pinfo['pid']] = pinfo
            except (psutil.NoSuchProcess, psutil.AccessDenied, psutil.ZombieProcess):
                pass
        process_names = []
        for pid, process in processes.items():
            process_names.append(process['name'])
        return list(set(process_names))

    def find_main_process_pid(self, process_name) -> None:
        '''
        通过进程名找到进程名的主进程pid
        :param process_name:
        :return:
        '''
        for process in psutil.process_iter(['pid', 'name']):
            if process.info['name'] == process_name:
                parent = process.parent()
                if parent is None or parent.name() != process_name:
                    return process.info['pid']
        return None

    def get_child_pids(self, parent_id) -> list:
        '''
        通过主进程pid获取所有子进程pid
        :param parent_id:
        :return:
        '''
        parent_process = psutil.Process(parent_id)
        return [child_process.pid for child_process in parent_process.children]

    def get_pid_find_pid(self, pid) -> None:
        '''
        通过pid获取进程名
        :param pid:
        :return:
        '''
        try:
            process = psutil.Process(pid)
            name = process.name()
            return name
        except psutil.NoSuchProcess:
            return None

    def get_pids_process(self, pids) -> list:
        '''
        获取应用内所有pids，包含父进程及子进程 process
        :param pids:
        :return:
        '''
        subprocesses = set()
        for proc in psutil.process_iter():
            try:
                pinfo = proc.as_dict(attrs=['pid', 'name', 'ppid'])
                # 如果进程的父进程ID在PID列表中，则认为它是指定应用程序的子进程
                if pinfo['pid'] in pids or pinfo['ppid'] in pids:
                    subprocesses.add(proc)
            except (psutil.NoSuchProcess, psutil.AccessDenied, psutil.ZombieProcess):
                pass
        return subprocesses

    def get_children_pids(self, pid):
        '''
        通过主进程获取所有子进程pid
        :param pid:
        :return:
        '''
        children = set()
        try:
            parent_process = psutil.Process(pid)
            for child in parent_process.children(recursive=True):
                children.add(child.pid)
        except psutil.NoSuchProcess:
            # print(f"主进程 {pid} 不存在或已终止.")
            pass
        except Exception as e:
            # print(f"获取子进程PID时发生错误: {e}")
            pass
        return children

    def find_main_process_pid(self, process_name) -> None:
        '''
        通过进程名找到进程名的主进程pid
        :param process_name:
        :return:
        '''
        for process in psutil.process_iter(['pid', 'name']):
            if process.info['name'] == process_name:
                parent = process.parent()
                if parent is None or parent.name() != process_name:
                    return process.info['pid']
        return None



try:
    import wmi
except:
    pass
import time
import ctypes
import asyncio
import psutil

# 定义Win32 API函数
GetLastError = ctypes.windll.kernel32.GetLastError
GetCurrentProcess = ctypes.windll.kernel32.GetCurrentProcess
OpenProcess = ctypes.windll.kernel32.OpenProcess
GetSystemTimes = ctypes.windll.kernel32.GetSystemTimes
GetProcessTimes = ctypes.windll.kernel32.GetProcessTimes
GetProcessIoCounters = ctypes.windll.kernel32.GetProcessIoCounters
PROCESS_VM_READ = 0x0010
# 权限
PROCESS_QUERY_INFORMATION = 0x0400


class WinPerformance:

    def __init__(self, pid, interval=1):
        if pid is None or not psutil.pid_exists(pid):
            print(f'进程{pid}不存在，停止监测')
            return
        self.pid = pid
        self.interval = interval
        self.app_cpu_percent = 0
        self.system_cpu_percent = 0
        self.memory_private = 0
        self.memory_workset = 0
        self.memory_percent = 0
        self.thread_count = 0
        self.io_read_speed = 0
        self.io_write_speed = 0
        self.header = ['时间',
                       'App CPU/%',
                       'System CPU/%',
                       '内存(Private)/MB',
                       '内存(WorkSet)/MB',
                       'App内存使用率/%',
                       '线程数',
                       'IO(Read)/b',
                       'IO(Write)/b',
                       ]

    def get_cpu_percent(self):
        hProcess = OpenProcess(PROCESS_QUERY_INFORMATION, False, self.pid)
        if hProcess == 0:
            print(f"Error: {GetLastError()}")
            return None
        FILETIME = ctypes.c_longlong * 2
        lpCreationTime, lpExitTime = FILETIME(), FILETIME()
        lpKernelTime, lpUserTime = FILETIME(), FILETIME()
        lpIdleTime, lpKernelTime2 = FILETIME(), FILETIME()
        # 获取前1s数据
        GetProcessTimes(hProcess, ctypes.byref(lpCreationTime), ctypes.byref(lpExitTime),
                        ctypes.byref(lpKernelTime), ctypes.byref(lpUserTime))
        GetSystemTimes(ctypes.byref(lpIdleTime), ctypes.byref(lpKernelTime2), None)
        kernel_time_start = lpKernelTime[0] + (lpKernelTime[1] << 32)
        user_time_start = lpUserTime[0] + (lpUserTime[1] << 32)
        idle_time_start = lpIdleTime[0] + (lpIdleTime[1] << 32)

        io_counters_start = self.get_io_counters()
        time.sleep(self.interval)
        # 获取当前数据
        GetProcessTimes(hProcess, ctypes.byref(lpCreationTime), ctypes.byref(lpExitTime),
                        ctypes.byref(lpKernelTime), ctypes.byref(lpUserTime))
        GetSystemTimes(ctypes.byref(lpIdleTime), ctypes.byref(lpKernelTime2), None)
        kernel_time_end = lpKernelTime[0] + (lpKernelTime[1] << 32)
        user_time_end = lpUserTime[0] + (lpUserTime[1] << 32)
        idle_time_end = lpIdleTime[0] + (lpIdleTime[1] << 32)
        # CPU总时间
        total_time = (kernel_time_end - kernel_time_start) + (user_time_end - user_time_start)
        # 系统空闲时间
        idle_time = idle_time_end - idle_time_start
        # 计算进程占用的CPU使用率（百分比）
        self.app_cpu_percent = round(100.0 * total_time / (total_time + idle_time), 2)  # app cpu/%
        system_times = psutil.cpu_times_percent()
        self.system_cpu_percent = round(100 - system_times.idle, 2)  # system cpu/%

        if io_counters_start:
            io_counters_end = self.get_io_counters()
            if io_counters_end:
                self.io_read_speed = (
                                             io_counters_end.ReadTransferCount - io_counters_start.ReadTransferCount) / self.interval
                self.io_write_speed = (
                                              io_counters_end.WriteTransferCount - io_counters_start.WriteTransferCount) / self.interval

        return self.app_cpu_percent, self.system_cpu_percent, self.io_read_speed, self.io_write_speed

    def get_memory_usage(self):
        w = wmi.WMI()
        process = w.Win32_Process(ProcessId=self.pid)[0]
        self.memory_private = round(int(process.PrivatePageCount) / 1024 / 1024, 2)  # MB
        self.memory_workset = round(int(process.WorkingSetSize) / 1024 / 1024, 2)  # MB
        try:
            process = psutil.Process(self.pid)
            self.memory_percent = u'%.2f' % process.memory_percent()  # 内存使用率
            self.thread_count = process.num_threads()  # 线程数
        except:
            self.memory_percent = 0
            self.thread_count = 0
        return self.memory_private, self.memory_workset, self.memory_percent, self.thread_count

    def get_io_counters(self):
        counters = IO_COUNTERS()
        hProcess = OpenProcess(PROCESS_QUERY_INFORMATION, False, self.pid)
        if hProcess == 0:
            error_code = ctypes.windll.kernel32.GetLastError()
            print(f"Error opening process (code: {error_code})")
            return None
        if not GetProcessIoCounters(hProcess, ctypes.byref(counters)):
            error_code = ctypes.windll.kernel32.GetLastError()
            print(f"Error getting IO counters (code: {error_code})")
            return None
        ctypes.windll.kernel32.CloseHandle(hProcess)
        return counters

    def get_all_usage(self):
        return [
            self.app_cpu_percent,  # 0
            self.system_cpu_percent,  # 1
            self.memory_private,  # 2
            self.memory_workset,  # 3
            self.memory_percent,  # 6
            self.thread_count,  # 7
            self.io_read_speed,  # 4
            self.io_write_speed,  # 5
        ]

    async def get_all_monitor(self):
        await asyncio.gather(self.get_cpu_percent(), self.get_memory_usage())
        return self.get_all_usage()


class IO_COUNTERS(ctypes.Structure):
    _fields_ = [('ReadOperationCount', ctypes.c_ulonglong),
                ('WriteOperationCount', ctypes.c_ulonglong),
                ('OtherOperationCount', ctypes.c_ulonglong),
                ('ReadTransferCount', ctypes.c_ulonglong),
                ('WriteTransferCount', ctypes.c_ulonglong),
                ('OtherTransferCount', ctypes.c_ulonglong)]



from setuptools import setup, find_packages

with open('README.md', 'r') as f:
    long_description = f.read()

setup(name='pmonitor',
      version='1.5.5',
      description='pc monitor',
      long_description=long_description,
      author='cfr',
      author_email='1354592998@qq.com',
      install_requires=[
            'PyLibreHardwareMonitor>=1.2.2',
            'psutil>=5.9.8'
      ],
      license='MIT',
      packages=find_packages(),
      package_data={
          'monitor': ['DLL/*'],
      },
      platforms=['all'],
      classifiers=[],
      python_requires='>=3.6',

      entry_points={
          'console_scripts': ['pmonitor=monitor.run_monitor:main']  # 增加命令行指令运行的参数设置
      },

      )



import psutil
import time
import json
from monitor.get_process_name import *
import sys
import asyncio
import platform
import ctypes
import os
from PyLibreHardwareMonitor.computer import Computer

if platform.system() != 'Windows':
    from monitor.mac_gpu import get_gpu_memory


class PidsPerf:

    def __init__(self, process_name, interval=1):
        self.process_name = process_name
        self.interval = interval
        self.processUtil = ProcessName()

    def get_mac_perf(self):
        current_pid = self.processUtil.find_main_process_pid(self.process_name)
        while True:
            minor_cpu_sum = 0
            minor_real_mem_sum = 0
            minor_mem_percent_sum = 0
            minor_vss_mem_sum = 0
            minor_thread_count_sum = 0
            gpu_memory_usage = 0
            gpu_memory_total = 0
            gpu_memory_free = 0
            current_pids = self.processUtil.get_children_pids(current_pid)  # 获取所有子进程
            current_pids.add(current_pid)  # 将主进程及子进程添加到list中
            pids_process = self.processUtil.get_pids_process(current_pids)  # 获取所有进程

            mem_info = psutil.virtual_memory()
            mem_total = round(mem_info.total / 1024 / 1024, 2)  # 总虚拟内存
            mem_available = round(mem_info.available / 1024 / 1024, 2)  # 剩余虚拟内存

            gpu_memory_usage, gpu_memory_total, gpu_memory_free = get_gpu_memory()
            for process in pids_process:
                try:
                    cpu_percent = process.cpu_percent()
                    process_memory = process.memory_info()
                    mem_percent = u'%.2f' % (process.memory_percent())  # 内存利用率
                    real_mem = round(process_memory.rss / 1024 / 1024, 2)  # 实际内存
                    vss_mem = round(process_memory.vms / 1024 / 1024, 2)  # 虚拟内存
                    thread_count = process.num_threads()  # 线程总数
                except (psutil.NoSuchProcess, psutil.AccessDenied, psutil.ZombieProcess):
                    cpu_percent = 0
                    mem_percent = 0
                    real_mem = 0
                    thread_count = 0
                    vss_mem = 0

                # 将性能数据保存到对应PID的数据列表中
                if process.pid == current_pid:  # 主进程数据处理
                    main_cpu = cpu_percent
                    main_real_mem = real_mem
                    main_mem_percent = round(float(mem_percent), 2)
                    main_vss_mem = vss_mem
                    main_thread_count = thread_count
                    main_data = {'cpu': main_cpu, 'memory': main_real_mem, 'memory percent': main_mem_percent,
                                 'thread count': main_thread_count, 'vss memory': main_vss_mem}
                    # print('主进程', pid_name, main_cpu, main_real_mem, main_mem_percent, main_thread_count)
                else:  # 子进程数据处理
                    minor_cpu_sum += cpu_percent
                    minor_real_mem_sum += real_mem
                    minor_vss_mem_sum += vss_mem
                    minor_mem_percent_sum += float(mem_percent)
                    minor_thread_count_sum += thread_count
                # pid_data[process.pid].append((cpu_percent, real_mem, mem_percent, thread_count))
            minor_data = {'cpu': round(float(minor_cpu_sum), 2), 'memory': round(float(minor_real_mem_sum), 2),
                          'memory percent': round(float(minor_mem_percent_sum), 2),
                          'thread count': minor_thread_count_sum, 'vss memory': minor_vss_mem_sum}
            # print('其他子进程', pid_name, minor_cpu_sum, minor_real_mem_sum, minor_mem_percent_sum,
            #       minor_thread_count_sum)
            # 获取磁盘IO
            io_read_bytes_start, io_write_bytes_start = self.get_disk_usage()
            time.sleep(self.interval)
            io_read_bytes_end, io_write_bytes_end = self.get_disk_usage()  # io read/write
            io_read_bytes = io_read_bytes_end - io_read_bytes_start  # io read/byte
            io_write_bytes = io_write_bytes_end - io_write_bytes_start  # io write/byte
            disk_data = {'io read': io_read_bytes, 'io write': io_write_bytes}
            gpu_data = {'gpu memory usage': gpu_memory_usage, 'gpu memory total': gpu_memory_total,
                        'gpu memory free': gpu_memory_free}
            data = {'main': main_data, 'other': minor_data, 'disk': disk_data, "gpu": gpu_data,
                    "memory_total": mem_total, "memory_available": mem_available}
            json_data = json.dumps(data)
            print(json_data)
            sys.stdout.flush()

    def get_disk_usage(self):
        '''
        Mac 获取进程磁盘读写情况
        :return:
        '''
        try:
            io_counters = psutil.disk_io_counters()
            io_read_bytes = io_counters.read_bytes
            io_write_bytes = io_counters.write_bytes
            return io_read_bytes, io_write_bytes
        except (psutil.NoSuchProcess, psutil.AccessDenied, psutil.ZombieProcess):
            pass
        return 0, 0

    def get_win_perf(self):
        from monitor.WinPidUtil import PidWinPerformance, FILETIME
        from ctypes import CDLL, c_int, c_char_p, byref
        pidWinPerf = PidWinPerformance()
        system_time_pre = pidWinPerf.filetime_to_100nano_seconds(FILETIME())
        p_pid = self.processUtil.find_main_process_pid(self.process_name)
        child_pids = self.processUtil.get_children_pids(p_pid)
        child_pids.add(p_pid)  # app内所有pid

        process_cpu_times_pre = {pid: pidWinPerf.get_process_cpu_time(pid) for pid in child_pids}
        process_io_counters_pre = {pid: pidWinPerf.get_io_counters(pid) for pid in child_pids}

        dll_path = os.path.join(os.path.dirname(__file__), 'DLL', 'GpuMonitorLib.dll')
        gpu_monitor_dll = ctypes.CDLL(dll_path)
        gpu_monitor_dll.GetGPUDataForProcess.argtypes = [ctypes.c_ulong]
        gpu_monitor_dll.GetGPUDataForProcess.restype = ctypes.c_char_p
        # 获取GPU物理内存
        # computer = Computer()
        # gpu_key = list(computer.gpu.keys())[0]
        while True:
            minor_cpu_sum = 0
            minor_workSet_mem_sum = 0
            minor_private_mem_sum = 0
            minor_mem_percent_sum = 0
            minor_vss_mem_sum = 0
            minor_thread_count_sum = 0
            minor_io_read_sum = 0
            minor_io_write_sum = 0
            minor_gpu_dedicated_sum = 0  # （专用内存）
            minor_gpu_system_sum = 0  # (共享内存)
            minor_gpu_committed_sum = 0  # (总使用内存)
            minor_gpu_percent_sum = 0  # (GPU内存占比)
            main_gpu_dedicated_sum = 0  # （专用内存）
            main_gpu_system_sum = 0  # (共享内存)
            main_gpu_committed_sum = 0  # (总使用内存)
            main_gpu_percent_sum = 0  # (GPU内存占比)

            start_time = time.time()  # 记录循环开始的时间
            system_idle_time = FILETIME()
            system_kernel_time = FILETIME()
            system_user_time = FILETIME()
            ctypes.windll.kernel32.GetSystemTimes(byref(system_idle_time),
                                                  byref(system_kernel_time),
                                                  byref(system_user_time))
            system_time_post = pidWinPerf.filetime_to_100nano_seconds(
                system_kernel_time) + pidWinPerf.filetime_to_100nano_seconds(system_user_time)
            p_pid = self.processUtil.find_main_process_pid(self.process_name)
            child_pids = self.processUtil.get_children_pids(p_pid)
            child_pids.add(p_pid)  # app内所有pid
            child_pids = {pid for pid in child_pids if psutil.pid_exists(pid)}  # 移除不存在的进程

            # small_data = computer.gpu[gpu_key]['SmallData']  # 获取GPU物理内存信息
            try:
                new_pids_cpu_time_pre = {pid: pidWinPerf.get_process_cpu_time(pid) for pid in child_pids if
                                         pid not in process_cpu_times_pre}
                process_cpu_times_pre.update(new_pids_cpu_time_pre)
                new_process_io_counters_pre = {pid: pidWinPerf.get_io_counters(pid) for pid in child_pids}
                process_io_counters_pre.update(new_process_io_counters_pre)
            except psutil.Error as ex:
                print(f"Error updating counters: {ex}")

            # print('----------------------------------------------------------')
            # print('pid', child_pids)
            memory_total = round(psutil.virtual_memory().total / 1024 / 1024, 2)
            memory_available = round(psutil.virtual_memory().available / 1024 / 1024, 2)

            for pid in child_pids:
                try:
                    process = psutil.Process(pid)
                    pid_name = process.name()
                    mem_vss = round(process.memory_info().vms / 1024 / 1024, 2)
                    mem_percent = round(process.memory_percent(), 2)
                    thread_count = process.num_threads()
                except (psutil.AccessDenied, psutil.NoSuchProcess, psutil.ZombieProcess):
                    # print('pid is not found')
                    mem_percent = 0
                    thread_count = 0
                    mem_vss = 0
                process_cpu_time_post = pidWinPerf.get_process_cpu_time(pid)
                memory_info = pidWinPerf.get_process_memory(pid)
                io_counters = pidWinPerf.get_io_counters(pid)
                if io_counters is None or pid not in process_io_counters_pre:
                    continue
                read_bytes_sec = (io_counters.ReadTransferCount - process_io_counters_pre[
                    pid].ReadTransferCount) / self.interval
                write_bytes_sec = (io_counters.WriteTransferCount - process_io_counters_pre[
                    pid].WriteTransferCount) / self.interval
                gpu_info_json = gpu_monitor_dll.GetGPUDataForProcess(pid).decode()
                gpu_info = json.loads(gpu_info_json)
                if process_cpu_time_post:
                    # 计算CPU使用率
                    cpu_usage = round((process_cpu_time_post - process_cpu_times_pre[pid]) / (
                            system_time_post - system_time_pre) * 100, 2)
                    # 更新之前的CPU时间
                    process_cpu_times_pre[pid] = process_cpu_time_post
                    workSet_mem = round(memory_info.WorkingSetSize / 1024 / 1024, 2)  # MB
                    private_mem = round(memory_info.PrivateUsage / 1024 / 1024, 2)  # MB
                    if pid == p_pid:

                        if gpu_info is not None:
                            for item in gpu_info:
                                if item['ProcessID'] == p_pid:
                                    main_gpu_dedicated_sum += item['GPUProcessMemoryLocalUsage']  # （专用内存）
                                    main_gpu_system_sum += item['GPUProcessMemorySharedUsage']  # (共享内存)
                                    main_gpu_committed_sum += item['GPUProcessMemoryTotalCommitted']  # (总使用内存)
                                    main_gpu_percent_sum += item['GPUUtilizationPercent']  # (GPU内存占比)
                            # print('主进程pid：', item['ProcessID'], 'Dedicated:',
                            #       main_gpu_dedicated_sum,
                            #       'System:', main_gpu_system_sum,
                            #       'Committed:', main_gpu_committed_sum, 'GPU Usage:',
                            #       main_gpu_percent_sum)
                        main_data = {'cpu': cpu_usage, 'private': private_mem, "workset": workSet_mem,
                                     "vss memory": mem_vss,
                                     "mem percent": mem_percent,
                                     "thread count": thread_count,
                                     'gpu_Dedicated': round(float(main_gpu_dedicated_sum), 2),
                                     'gpu_System': round(float(main_gpu_system_sum), 2),
                                     'gpu_Committed': round(float(main_gpu_committed_sum), 2),
                                     'gpu_Usage': round(float(main_gpu_percent_sum), 2)}

                        # print(
                        #     f'主进程数据：cpu：{cpu_usage}，workSet：{workSet_mem}，private：{private_mem}，mem_percent：{mem_percent}，thread_count：{thread_count}')
                    else:
                        minor_cpu_sum += cpu_usage  # 子进程总
                        minor_vss_mem_sum += mem_vss  # 子进程vss内存
                        minor_workSet_mem_sum += workSet_mem  # 子进程workSet内存
                        minor_private_mem_sum += private_mem  # 子进程private内存
                        minor_mem_percent_sum += mem_percent  # 子进程内存使用率
                        minor_thread_count_sum += thread_count  # 子进程线程总数
                        if gpu_info is not None:
                            for item in gpu_info:
                                if item['ProcessID'] != p_pid:
                                    # print('子进程pid：', item['ProcessID'], 'Dedicated:',
                                    #       item['GPUProcessMemoryDedicatedUsage'],
                                    #       'System:', item['GPUProcessMemorySharedUsage'],
                                    #       'Committed:', item['GPUProcessMemoryTotalCommitted'], 'GPU Usage:',
                                    #       item['GPUUtilizationPercent'])
                                    minor_gpu_dedicated_sum += item['GPUProcessMemoryLocalUsage']
                                    minor_gpu_system_sum += item['GPUProcessMemorySharedUsage']
                                    minor_gpu_committed_sum += item['GPUProcessMemoryTotalCommitted']
                                    minor_gpu_percent_sum += item['GPUUtilizationPercent']
                    minor_io_read_sum += read_bytes_sec  # 所有进程IO读取速率总数
                    minor_io_write_sum += write_bytes_sec  # 所有进程IO写入速率总数

                else:
                    # print(f"PID {pid}: Process not found or access denied")
                    continue
            disk_data = {'io read': minor_io_read_sum, 'io write': minor_io_write_sum}
            other_data = {'cpu': minor_cpu_sum, 'private': round(float(minor_private_mem_sum), 2),
                          'workset': round(float(minor_workSet_mem_sum), 2),
                          'vss memory': minor_vss_mem_sum,
                          'memory percent': round(float(minor_mem_percent_sum), 2),
                          'thread count': minor_thread_count_sum,
                          'gpu_Dedicated': round(minor_gpu_dedicated_sum, 2),
                          'gpu_System': round(minor_gpu_system_sum, 2),
                          'gpu_Committed': round(minor_gpu_committed_sum, 2),
                          'gpu_Usage': round(minor_gpu_percent_sum, 2)}
            # if 'GPU Memory Used' in small_data:
            #     gpu_data = {'gpu memory usage': round(small_data['GPU Memory Used'], 2),
            #                 'gpu memory total': round(small_data['GPU Memory Total'], 2),
            #                 'gpu memory free': round(small_data['GPU Memory Free'], 2)}
            # else:
            #     gpu_data = {'gpu memory usage': round(small_data['D3D Shared Memory Used'], 2),
            #                 'gpu memory total': round(small_data['D3D Shared Memory Total'], 2),
            #                 'gpu memory free': round(small_data['D3D Shared Memory Free'], 2)}
            try:
                data = {'main': main_data, 'other': other_data, 'disk': disk_data,
                        # 'gpu': gpu_data,
                        'memory_total': memory_total,
                        'memory_available': memory_available}
                json_data = json.dumps(data)
                print(json_data)
            except UnboundLocalError:
                continue
            sys.stdout.flush()
            # 更新系统时间
            system_time_pre = system_time_post
            # 等待以确保大约每秒更新一次
            elapsed = time.time() - start_time
            time.sleep(max(0, self.interval - elapsed))

    async def start_perf(self):
        if platform.system() == 'Windows':
            await asyncio.gather(self.get_win_perf())
        else:
            await asyncio.gather(self.get_mac_perf())
